@@ -1,6 +1,5 @@
-// MAKE SURE THE GLOW LIBRARY HAS THE CORRECT VERSION NUMBER:
 define(["nbextensions/vpython_libraries/jquery-ui.custom.min",
-        "nbextensions/vpython_libraries/glow.2.1.min"], function() {
+        "nbextensions/vpython_libraries/glow.min"], function() {
 
 // The following is necessary to be able to re-run programs.
 // Otherwise the repeated execution of update_canvas() causes problems after klling Python.
@@ -10,15 +9,18 @@ function msclock() {
     if (performance.now) return performance.now()
     else return new Date().getTime()
 }
+var tstart = msclock()
+
+window.Jupyter_VPython = requirejs.s.contexts._.config.paths.nbextensions + '/vpython_data/'
 
 function fontloading() {
-    var fsans = '/nbextensions/vpython_data/Roboto-Medium.ttf'
+    var fsans = requirejs.s.contexts._.config.paths.nbextensions + '/vpython_data/Roboto-Medium.ttf'
     opentype_load(fsans, function(err, fontrefsans) {
         if (err) throw new Error('Font ' + fsans + ' could not be loaded: ' + err)
         window.__font_sans = fontrefsans // an opentype.js Font object
         console.log('SANS-SERIF FONT LOADED')
     })
-    var fserif = '/nbextensions/vpython_data/NimbusRomNo9L-Med.otf'
+    var fserif = requirejs.s.contexts._.config.paths.nbextensions + '/vpython_data/NimbusRomNo9L-Med.otf'
     opentype_load(fserif, function(err, fontrefserif) {
         if (err) throw new Error('Font ' + fserif + ' could not be loaded: ' + err)
         window.__font_serif = fontrefserif // an opentype.js Font object
@@ -105,6 +107,9 @@ var interval = 30 // milliseconds
 
 function update_canvas() { // mouse location and other stuff
     "use strict";
+    //var t = msclock()
+    //console.log(t-tstart)
+    //tstart = t
     for (var ss in sliders){
         comm.send( { arguments: [ sliders[ss] ] } )
     }
@@ -154,6 +159,12 @@ function send_pick(cvs, p, seg) {
     comm.send( {arguments: [evt]} ) 
 }
 
+function send_compound(cvs, pos, size) {
+    "use strict";
+    var evt = {event: '_compound', 'canvas': cvs, 'pos': [pos.x, pos.y, pos.z], 'size': [size.x, size.y, size.z]}
+    comm.send( {arguments: [evt]} )
+}
+
 // attrs are X in {'a': '23X....'}
 var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use single and double quotes; available: +-, 
          'e':'ambient', 'f':'axis', 'g':'size', 'h':'origin', 'i':'textcolor',
@@ -184,7 +195,7 @@ var attrs = {'a':'pos', 'b':'up', 'c':'color', 'd':'trail_color', // don't use s
 var attrsb = {'a':'userzoom', 'b':'userspin', 'c':'range', 'd':'autoscale', 'e':'fov',
               'f':'normal', 'g':'data', 'h':'checked', 'i':'disabled', 'j':'selected',
               'k':'vertical', 'l':'min', 'm':'max', 'n':'step', 'o':'value',
-              'p':'left', 'q':'right', 'r':'top', 's':'bottom'}
+              'p':'left', 'q':'right', 'r':'top', 's':'bottom', 't':'_cloneid'}
 
 // methods are X in {'m': '23X....'}
 var methods = {'a':'select', 'c':'start', 'd':'stop', 'f':'clear', // unused bsxyCDFghzAB
@@ -210,9 +221,8 @@ function decode(data) { // [ [{'a': '3c0.0,1.0,1.0'}], .....] or can be a method
     var output = [], s, m, idx, attr, val, datatype, out
     for (var i in data) { // step through the list of dictionaries
         var d = data[i]
-        //console.log('---- in ------')
         // constructor or appendcmd not currently compressed; complex methods or attributes:
-        if (d['_pass'] !== undefined) { 
+        if (d['_pass'] !== undefined) {
             delete d['_pass']
             output.push(d)
             continue
@@ -226,7 +236,6 @@ function decode(data) { // [ [{'a': '3c0.0,1.0,1.0'}], .....] or can be a method
             s = d['m']
             datatype = 'method'
         }
-        //for (var a in d) console.log(a, d[a])
         m = s.match(patt)
         idx = Number(m[1])
         if (datatype == 'attr') {
@@ -317,6 +326,9 @@ function handler(msg) {
                                     glowObjs[cmd.idx][cmd.attr] = v
                                 }
                             }
+                        } else if (cmd.attr == 'lights') {
+                            if (cmd.val == 'empty_list') cmd.val = []
+                            glowObjs[cmd.idx][cmd.attr] = cmd.val
                         } else {
                             if (triangle_quad.indexOf(cmd.attr) !== -1) {
                                 glowObjs[cmd.idx][cmd.attr] = glowObjs[cmd.val]
@@ -326,7 +338,7 @@ function handler(msg) {
                         }
                     }
                     if (cmd.method !== undefined) {
-                        //console.log('cmd.method', cmd.method, cmd.cmd, cmd.val)
+                        //console.log('cmd.method', cmd.idx, cmd.method, cmd.val)
                         var parametric = ['splice', 'modify']
                         var val = cmd.val
                         if (cmd.method == 'GSprint') {
@@ -416,8 +428,8 @@ function handler(msg) {
                             } else {
                                cfg[attr.attr] = o2vec3(attr.value)
                             }                            
-                        } else if (attr.attr ==='pos' && (cmd.cmd === 'curve' || cmd.cmd === 'points' ||
-                                    cmd.cmd === 'extrusion')) {
+                        } else if ( (attr.attr == 'pos' && (cmd.cmd == 'curve' || cmd.cmd == 'points')) ||
+                                    (attr.attr == 'path' && cmd.cmd == 'extrusion') ) {
                             var ptlist = []
                             for (var kk = 0; kk < attr.value.length; kk++) {
                                 ptlist.push( o2vec3(attr.value[kk]) )
@@ -440,6 +452,9 @@ function handler(msg) {
                                     objects[k] = glowObjs[attr.value[k]]
                                 }
                             }
+                        } else if (attr.attr == "lights") {
+                            if (attr.value == 'empty_list') attr.value = []
+                            cfg[attr.attr] = attr.value
                         } else {
                             cfg[attr.attr] = attr.value
                             // console.log(attr.attr, attr.value)
@@ -499,11 +514,18 @@ function handler(msg) {
                         } else if (cmd.cmd === 'ellipsoid') {
                             glowObjs[cmd.idx] = sphere(cfg)
                         } else if (cmd.cmd === 'extrusion') {
-                            glowObjs[cmd.idx] = extrusion(cfg)
+                            var obj = glowObjs[cmd.idx] = extrusion(cfg)
+                            // Return computed compound pos and size to Python
+                            send_compound(obj.canvas['idx'], obj.pos, obj.size)
                         } else if (cmd.cmd === 'text') {
-                            glowObjs[cmd.idx] = text(cfg)
-                        } else if (cmd.cmd === 'lights') {
-                            glowObjs[cmd.idx] = lights(cfg)
+                            if (cfg._cloneid !== undefined) {
+                                var idoriginal = cfg._cloneid
+                                delete cfg._cloneid
+                                glowObjs[cmd.idx] = glowObjs[idoriginal].clone(cfg)
+                            } else {
+                                var obj = glowObjs[cmd.idx] = text(cfg)
+                                send_compound(obj.canvas['idx'], vec(obj.length, obj.descender, 0), vec(0,0,0))
+                            }
                         } else if (cmd.cmd === 'rotate') {
                             glowObjs[cmd.idx].rotate(cfg)
                         } else if (cmd.cmd === 'local_light') {
@@ -511,7 +533,16 @@ function handler(msg) {
                         } else if (cmd.cmd === 'distant_light') {
                             glowObjs[cmd.idx] = distant_light(cfg)
                         } else if (cmd.cmd === 'compound') {
-                            glowObjs[cmd.idx] = compound(objects, cfg)
+                            if (cfg._cloneid !== undefined) {
+                                var idoriginal = cfg._cloneid
+                                delete cfg._cloneid
+                                obj = glowObjs[cmd.idx] = glowObjs[idoriginal].clone(cfg)
+                            } else {
+                                glowObjs[cmd.idx] = compound(objects, cfg)
+                                var obj = glowObjs[cmd.idx]
+                                // Return computed compound pos and size to Python
+                                send_compound(obj.canvas['idx'], obj.pos, obj.size)
+                            }
                         } else if (cmd.cmd === 'canvas') {
                             glowObjs[cmd.idx] = canvas(cfg)
                             glowObjs[cmd.idx]['idx'] = cmd.idx
